//////////////////////////////////////////////////////////////////
//This is a simple demo of using naming service to store some information.
//You can read some details in http://java.sun.com/j2se/1.5.0/docs/guide/idl/jidlNaming.html
//However, I personally feel it is not very easy to understand.
// I assume we use only the "root" namingcontext to search by calling its "list" function.
// And the namecomponent is designed as brokername + ".hostname" as its "id" and the real
// host name is used as "kind". Similarly, brokername+".portnumber" as its "id" and the 
// port number is used as "kind". Therefore by giving the broker name, we can find its
// host name and port number. Of course you can also register the "Broker" object reference
// by using name like "broker" etc.
////////////////////////////////////////////////////////////////



import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POA;
import java.util.Properties;
import org.omg.CORBA.ORB.*;
import org.omg.CORBA.*;
import java.net.*;


public class NamingDemo
{	
	
	public static void main(String args[]) 
	{		
		try
		{
			ORB orb = ORB.init(args, null);
		
			// get the root naming context
			// NameServiceinvokes the transient name service
			org.omg.CORBA.Object objRef=
				orb.resolve_initial_references("NameService");

		
			NamingContextExt rootRef= NamingContextExtHelper.narrow(objRef);

			
			clearRegistry(rootRef);
			//System.out.println("unregister ok");
				
			registerBroker(rootRef, "broker101", "nickATI", 5001);
			//System.out.println("register ok");

			readContext(rootRef, "broker101");	
			
			//System.out.println("read ok");	
		
		}
		catch(Exception er)
		{
			System.out.println("error");
			System.out.println(er.getMessage());
		}
	}

	public static void registerBroker(NamingContextExt ncRef, String brokerName, String hostName, int portNumber)
	{
		try
		{
			NameComponent host[]={new NameComponent(brokerName+".hostname", hostName)};
			NameComponent port[]={new NameComponent(brokerName+".portnumber", Integer.toString(portNumber))};
			
			ncRef.rebind_context(host, ncRef);
			ncRef.rebind_context(port, ncRef);
		}
		catch (Exception er)
		{
			System.out.println(er);
		}
	}

	public static void readContext(NamingContextExt rootRef, String brokerName)
	{
		try
		{
			//org.omg.CORBA.Object brokerObj=rootRef.resolve_str(brokerName);

			//NamingContextExt brokerContext=(NamingContextExt)NamingContextHelper.narrow(brokerObj);

			BindingListHolder bl = new BindingListHolder();
			BindingIteratorHolder blIt= new BindingIteratorHolder();
			rootRef.list(1000, bl, blIt);

			Binding bindings[] = bl.value;

			for (int i=0; i < bindings.length; i++) 
			{
				int lastIx = bindings[i].binding_name.length-1;

				// check to see if this is a naming context
				if (bindings[i].binding_type == BindingType.ncontext) 
				{
					if (bindings[i].binding_name[lastIx].id.compareTo(brokerName+".hostname")==0)
					{
						System.out.println( "host name is "+  bindings[i].binding_name[lastIx].kind);
					}
					if (bindings[i].binding_name[lastIx].id.compareTo(brokerName+".portnumber")==0)
					{
						System.out.println( "port number is "+  bindings[i].binding_name[lastIx].kind);
					}
				} 
				else 
				{
					System.out.println("Object: " + 
						bindings[i].binding_name[lastIx].id);
				}
			}
		}
		catch (Exception e)
		{
			e.printStackTrace(System.err);
		} 	
	}


	public static void clearRegistry(NamingContextExt ncRef)
	{
		BindingListHolder bl=new BindingListHolder();
		BindingIteratorHolder bh=new BindingIteratorHolder();

		ncRef.list(100, bl, bh);
		Binding bindings[]=bl.value;
		NameComponent nc[];//=n[0].binding_name;

		for (int i=0; i<bindings.length; i++)			
		{			
			//System.out.println("loop for " + i);				
			if (bindings[i].binding_type==BindingType.ncontext)
			{
				nc=bindings[i].binding_name;			
								
						
				try
				{
					System.out.println("try to unbind "+ nc);
					ncRef.unbind(nc);
					
				}
				catch (Exception er)
				{
					System.out.println("unbind error in clear host name"+nc);
				}					
				
			}
		}		
		
	}
}
				